<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;

class ProdLojaCadastroAutomatico extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    const dataSourceProdutosLoja = 'ger12300_cadloja_produtos_loja';
    const dataSourceProdutos = 'ger12300_cadloja_produtos';
    const dataSourceProdutosNivel = 'ger12300_cadloja_produtos_nivel';
    
    public function __construct ($crudService, $parameterBag, $entityManager, Manager $dataSourceManager, Services\ProdLojaCadastroAutomatico $cadastroService) {
        $this->crudService              = $crudService;
        $this->parameterBag             = $parameterBag;
        $this->entityManager            = $entityManager;
        $this->dataSourceManager        = $dataSourceManager;
        $this->cadastroService 	        = $cadastroService;
    }
    
    public function findProdutos(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceProdutosLoja;
        $this->parameterBagColumns = array('NRORG', 'CDFILIAL', 'CDLOJA');
        parent::find($request,$response);
    }
    
    public function findProdutosLoja (Request\Filter $request, Response $response) {
        $this->dataSourceName = self::dataSourceProdutos;
        $this->parameterBagColumns = array('NRORG', 'CDFILIAL', 'NIVELINICIAL', 'CDLOJA');
        parent::find($request,$response);
    }
    
    public function findProdutosLojaNivel (Request\Filter $request, Response $response) {
        $this->dataSourceName = self::dataSourceProdutosNivel;
        $this->parameterBagColumns = array('NRORG', 'CDFILIAL', 'CDLOJA');
        parent::find($request,$response);
    }
    
    public function salvarCadastroAutomatico (DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $tipoCadastro = $row['NMTPSELECT'];
        
        $CDFILIAL       = $row['CDFILIAL'];
        $CDLOJA         = $row['CDLOJA'];
        $NRORG          = $row['NRORG'];
        $NRCONFTELA     = $row['NRCONFTELA'];
        $NRSEQIMPRLOJA  = $row['NRSEQIMPRLOJA'];
        $CDAMBIENTE     = $row['CDAMBIENTE'];
        $DTINIVIGENCIA  = $row['DTINIVIGENCIA'];
        $arrayProdutos = [];
        
        if($tipoCadastro == 'I') {
            $CDPRODINI = $row['CDPRODUTOINICIAL'];
            $CDPRODFIN   = $row['CDPRODUTOFINAL'];
            $parametros = array(
                ':PRODINICIAL'  => $CDPRODINI,
                ':PRODFINAL'    => $CDPRODFIN,
                ':NRORG'        => $NRORG
            );
            $arrayProdutos = $this->cadastroService->getIntervaloProdutos($parametros);
        } else if ($tipoCadastro == 'L') {
            $arrayProdutos = $row['PRODUTOS'];
        } else if ($tipoCadastro == 'T') {
             $parametros = array(
                ':CDFILIAL'     => $CDFILIAL,
                ':CDLOJA'       => $CDLOJA,
                ':NRORG'        => $NRORG
            );
            $arrayProdutos = $this->cadastroService->getIntervaloTodosProdutos($parametros);
        } else {
            throw new Exception('Erro ao buscar produtos.');
        }
        
        $params = array(
			':CDFILIAL'       => $CDFILIAL,
			':CDLOJA'         => $CDLOJA,
			':NRCONFTELA'     => $NRCONFTELA,
			':CDAMBIENTE'     => $CDAMBIENTE,
			':NRSEQIMPRLOJA'  => $NRSEQIMPRLOJA,
			':NRORG'          => $NRORG,
			':DTINIVIGENCIA'  => $DTINIVIGENCIA
		);
		
		$this->cadastroService->cadastroAutomaticoInsert($params, $arrayProdutos);
		
        $response->addMessage(new Message("Cadastro automático realizado com sucesso.", 'SUCCESS'));
    }
    
    public function deleteMultiplo ($request, $response){
        try {
            $dataSet = $request->getDataSet();
            $this->dataSourceName = self::dataSourceProdutos;
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
            $nrDeletedRows = count($deletedRows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function deleteMultiploNivel ($request, $response){
        try {
            $dataSet = $request->getDataSet();
            $this->dataSourceName = self::dataSourceProdutosNivel;
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
            $nrDeletedRows = count($deletedRows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function deleteProduct(DTO\Request\Row $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceProdutos;
        parent::delete($request,$response);
    }
    
    public function deleteProductNivel(DTO\Request\Row $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceProdutosNivel;
        parent::delete($request,$response);
    }
}