<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;

class RelRentabilidadeProdContinuoService {

	public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function fetchReport($conditions){
        try{
            $params = self::buildQueryParams($conditions);
            $relatorio = self::GET_RENTABILIDADE_PROD;
            if($conditions['TIPORELATORIO'] == 'A'){
                $relatorio = self::GET_RENTABILIDADE_PROD;
            }else if($conditions['TIPORELATORIO'] == 'C'){
                $relatorio = self::GET_RENTABILIDADE_PROD_CONSO;
            }
            return $this->entityManager->getConnection()->fetchAll($relatorio, $params);
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    private function buildQueryParams($conditions){
        if(array_key_exists('CDFILIAL', $conditions)){
            $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
            $conditions['TCDFILIAL']= NULL;
        } else {
            $conditions['CDFILIAL'] = NULL;
            $conditions['TCDFILIAL']= 'T';
        }
        if(array_key_exists('CDLOJA', $conditions)){
            $conditions['CDLOJA'] = implode('_', $conditions['CDLOJA']);
            $conditions['TCDLOJA']= NULL;
        } else {
            $conditions['CDLOJA'] = NULL;
            $conditions['TCDLOJA']= 'T';
        }
        if(!isset($conditions['ALLFIEDLS'])){
            $conditions['ALLFIEDLS'] = 'T';
        }
        $params = array(
            ':TCDFILIAL'  => $conditions['TCDFILIAL'],
            ':CDFILIAL' => $conditions['CDFILIAL'],
            ':CDLOJA'   => $conditions['CDLOJA'],
            ':TCDLOJA'  => $conditions['TCDLOJA'],
            ':NRORG'    => $conditions['NRORG'],
            ':DTINI'    => $conditions['DTENTRVENDA'][0],
            ':DTFIN'    => $conditions['DTENTRVENDA'][1],
            ':TIPOCUSTO'=> $conditions['CDTIPOCUSTO'],
            ':ALLFIEDLS' => $conditions['ALLFIEDLS']
        );
        return $params;
    }
    
    const GET_RENTABILIDADE_PROD = " 
        SELECT *  FROM ( SELECT V.CDFILIAL, F.NMFILIAL, V.CDLOJA, L.NMLOJA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, I.VRUNITVEND, V.NRORG, SUM(I.QTPRODVEND) AS QTPRODVEND,
          SUM(M.VRIMPOPRODIT) AS VRIMPOSTO, 
          SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND,0) + NVL(I.VRACRITVEND,0)) AS VRTOTAL,
          SUM(CUSTO_COMPOSICAO(I.CDPRODUTO, V.CDFILIAL, V.CDFILIAL, :TIPOCUSTO, I.QTPRODVEND, V.CDLOJA, V.DTENTRVENDA)) AS VRCUSTO,
          SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND,0) + NVL(I.VRACRITVEND,0)) -
          SUM(M.VRIMPOPRODIT) - SUM(CUSTO_COMPOSICAO(I.CDPRODUTO, V.CDFILIAL, V.CDFILIAL, :TIPOCUSTO, I.QTPRODVEND, V.CDLOJA, V.DTENTRVENDA)) AS VRRENTA,
          V.CDFILIAL||' '||F.NMFILIAL AS LABELFILIAL, V.CDLOJA||' '||L.NMLOJA AS LABELLOJA
        FROM VENDA V
         JOIN ITEMVENDA I
          ON V.CDFILIAL = I.CDFILIAL
           AND V.CDCAIXA = I.CDCAIXA
           AND V.NRSEQVENDA = I.NRSEQVENDA
           AND V.NRORG = I.NRORG
         JOIN ITVENDAIMPOS M
          ON I.CDFILIAL = M.CDFILIAL 
           AND I.CDCAIXA = M.CDCAIXA
           AND I.NRSEQVENDA = M.NRSEQVENDA
           AND I.NRSEQUITVEND = M.NRSEQUITVEND
           AND I.NRORG = M.NRORG
         JOIN FILIAL F
          ON V.CDFILIAL = F.CDFILIAL
           AND V.NRORG = F.NRORG
         JOIN LOJA L
          ON V.CDFILIAL = L.CDFILIAL
           AND V.CDLOJA  = L.CDLOJA
           AND V.NRORG = L.NRORG
         JOIN PRODUTO P
          ON I.CDPRODUTO = P.CDPRODUTO
           AND I.NRORG = P.NRORG
        WHERE   
           V.IDSITUVENDA  = 'O'
           AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
           AND ((INSTR(:CDFILIAL  , V.CDFILIAL) <> 0) OR (:TCDFILIAL   = 'T'))
           AND ((INSTR(:CDLOJA  , V.CDFILIAL||V.CDLOJA) <> 0) OR (:TCDLOJA   = 'T'))
           AND V.NRORG = :NRORG
        GROUP BY
           V.CDFILIAL, F.NMFILIAL, V.CDLOJA, L.NMLOJA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, I.VRUNITVEND, V.NRORG, V.CDLOJA||' '||L.NMLOJA, V.CDFILIAL||' '||F.NMFILIAL
        ORDER BY
           V.CDFILIAL ASC, V.CDLOJA ASC, VRRENTA DESC)
           WHERE (UPPER(CDFILIAL||NMFILIAL||CDLOJA||NMLOJA||CDPRODUTO||NMPRODUTO||QTPRODVEND||VRUNITVEND||VRTOTAL||VRIMPOSTO||VRCUSTO||VRRENTA) like UPPER(:ALLFIEDLS)  OR :ALLFIEDLS = 'T' )
    ";
    
    const GET_RENTABILIDADE_PROD_CONSO = " 
         SELECT *  FROM ( SELECT B.CDPRODUTO, B.CDARVPROD, B.NMPRODUTO, 
           SUM(B.QTPRODVEND) AS QTPRODVEND,
          (SUM(B.VRUNITVEND)/B.QTDGRPROD) AS VRUNITVEND, 
          SUM(B.VRTOTAL) as VRTOTAL,
          SUM(B.VRIMPOSTO) AS VRIMPOSTO,
          (SUM(B.VRCUSTO)/SUM(B.QTPRODVEND)) AS VRCUSTO,
          (SUM(B.VRTOTAL) - (SUM(B.VRIMPOSTO) + SUM(B.VRCUSTO)/SUM(B.QTPRODVEND))) AS VRRENTA
          FROM (SELECT V.CDFILIAL, F.NMFILIAL, V.CDLOJA, L.NMLOJA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, I.VRUNITVEND, V.NRORG, SUM(I.QTPRODVEND) AS QTPRODVEND,
                    SUM(M.VRIMPOPRODIT) AS VRIMPOSTO, 
                    SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND,0) + NVL(I.VRACRITVEND,0)) AS VRTOTAL,
                    SUM(CUSTO_COMPOSICAO(I.CDPRODUTO, V.CDFILIAL, V.CDFILIAL, :TIPOCUSTO, I.QTPRODVEND, V.CDLOJA, V.DTENTRVENDA)) AS VRCUSTO,
                    SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND,0) + NVL(I.VRACRITVEND,0)) -
                    SUM(M.VRIMPOPRODIT) - SUM(CUSTO_COMPOSICAO(I.CDPRODUTO, V.CDFILIAL, V.CDFILIAL, :TIPOCUSTO, I.QTPRODVEND, V.CDLOJA, V.DTENTRVENDA)) AS VRRENTA,
                    count(*) over (PARTITION BY I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO) AS QTDGRPROD
                  FROM VENDA V
                   JOIN ITEMVENDA I
                    ON V.CDFILIAL = I.CDFILIAL
                     AND V.CDCAIXA = I.CDCAIXA
                     AND V.NRSEQVENDA = I.NRSEQVENDA
                     AND V.NRORG = I.NRORG
                   JOIN ITVENDAIMPOS M
                    ON I.CDFILIAL = M.CDFILIAL 
                     AND I.CDCAIXA = M.CDCAIXA
                     AND I.NRSEQVENDA = M.NRSEQVENDA
                     AND I.NRSEQUITVEND = M.NRSEQUITVEND
                     AND I.NRORG = M.NRORG
                   JOIN FILIAL F
                    ON V.CDFILIAL = F.CDFILIAL
                     AND V.NRORG = F.NRORG
                   JOIN LOJA L
                    ON V.CDFILIAL = L.CDFILIAL
                     AND V.CDLOJA  = L.CDLOJA
                     AND V.NRORG = L.NRORG
                   JOIN PRODUTO P
                    ON I.CDPRODUTO = P.CDPRODUTO
                     AND I.NRORG = P.NRORG
                  WHERE   
                     V.IDSITUVENDA  = 'O'
                     AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
                     AND ((INSTR(:CDFILIAL  , V.CDFILIAL) <> 0) OR (:TCDFILIAL   = 'T'))
                     AND ((INSTR(:CDLOJA  , V.CDFILIAL||V.CDLOJA) <> 0) OR (:TCDLOJA   = 'T'))
                     AND V.NRORG = :NRORG
                  GROUP BY
                     V.CDFILIAL, F.NMFILIAL, V.CDLOJA, L.NMLOJA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, I.VRUNITVEND, V.NRORG
                  ORDER BY
                     V.CDFILIAL ASC, V.CDLOJA ASC, VRRENTA DESC) B
            GROUP BY B.CDPRODUTO, B.CDARVPROD, B.NMPRODUTO, B.QTDGRPROD
            ORDER BY VRRENTA DESC )
           WHERE (UPPER(CDPRODUTO||NMPRODUTO||QTPRODVEND||VRUNITVEND||VRTOTAL||VRIMPOSTO||VRCUSTO||VRRENTA) like UPPER(:ALLFIEDLS)  OR :ALLFIEDLS = 'T' )
    ";
}